using System;
using System.Runtime.InteropServices;
using System.Collections;
using System.Reflection;

using SldWorks;
using SWPublished;
using SolidWorksTools.File;
using SwConst;
using System.Collections.Generic;


namespace SwCSharpGetSubAssyMassProps
{

    /// <summary>
    /// Summary description for SwCSharpGetSubAssyMassProps.
    /// </summary>
    [Guid("4bce671d-a638-456b-a81d-8811a44df524"), ComVisible(true)]
    [SwAddin(
        Description = "SwCSharpGetSubAssyMassProps description",
        Title = "SwCSharpGetSubAssyMassProps",
        LoadAtStartup = true
        )]
    public class SwAddin : SWPublished.ISwAddin
    {
        #region Local Variables
        ISldWorks iSwApp;
        ICommandManager iCmdMgr;
        List<IBody2> swBodies;
        int addinID;

        // Public Properties
        public ISldWorks SwApp
        {
            get { return iSwApp; }
        }
        public ICommandManager CmdMgr
        {
            get { return iCmdMgr; }
        }

        #endregion

        #region SolidWorks Registration
        [ComRegisterFunctionAttribute]
        public static void RegisterFunction(Type t)
        {

            #region Get Custom Attribute: SwAddinAttribute
            SwAddinAttribute SWattr = null;
            Type type = typeof(SwAddin);
            foreach (System.Attribute attr in type.GetCustomAttributes(false))
            {
                if (attr is SwAddinAttribute)
                {
                    SWattr = attr as SwAddinAttribute;
                    break;
                }
            }
            #endregion

            Microsoft.Win32.RegistryKey hklm = Microsoft.Win32.Registry.LocalMachine;
            Microsoft.Win32.RegistryKey hkcu = Microsoft.Win32.Registry.CurrentUser;

            string keyname = "SOFTWARE\\SolidWorks\\Addins\\{" + t.GUID.ToString() + "}";
            Microsoft.Win32.RegistryKey addinkey = hklm.CreateSubKey(keyname);
            addinkey.SetValue(null, 0);

            addinkey.SetValue("Description", SWattr.Description);
            addinkey.SetValue("Title", SWattr.Title);

            keyname = "Software\\SolidWorks\\AddInsStartup\\{" + t.GUID.ToString() + "}";
            addinkey = hkcu.CreateSubKey(keyname);
            addinkey.SetValue(null, Convert.ToInt32(SWattr.LoadAtStartup));
        }

        [ComUnregisterFunctionAttribute]
        public static void UnregisterFunction(Type t)
        {
            Microsoft.Win32.RegistryKey hklm = Microsoft.Win32.Registry.LocalMachine;
            Microsoft.Win32.RegistryKey hkcu = Microsoft.Win32.Registry.CurrentUser;

            string keyname = "SOFTWARE\\SolidWorks\\Addins\\{" + t.GUID.ToString() + "}";
            hklm.DeleteSubKey(keyname);

            keyname = "Software\\SolidWorks\\AddInsStartup\\{" + t.GUID.ToString() + "}";
            hkcu.DeleteSubKey(keyname);
        }

        #endregion

        #region ISwAddin Implementation
        public SwAddin()
        {
        }

        public bool ConnectToSW(object ThisSW, int cookie)
        {
            iSwApp = (ISldWorks)ThisSW;
            addinID = cookie;

            //Setup callbacks
            iSwApp.SetAddinCallbackInfo(0, this, addinID);

            #region Setup the Command Manager
            iCmdMgr = iSwApp.GetCommandManager(cookie);
            AddCommandMgr();
            #endregion

            return true;
        }

        public bool DisconnectFromSW()
        {
            RemoveCommandMgr();

            iSwApp = null;
            //The addin _must_ call GC.Collect() here in order to retrieve all managed code pointers 
            GC.Collect();
            return true;
        }
        #endregion

        #region UI Methods
        public void AddCommandMgr()
        {
            ICommandGroup cmdGroup;
            BitmapHandler iBmp = new BitmapHandler();
            Assembly thisAssembly;

            thisAssembly = System.Reflection.Assembly.GetAssembly(this.GetType());

            cmdGroup = iCmdMgr.CreateCommandGroup(1, "Get Mass Props", "Get Mass Props", "", -1);
            cmdGroup.LargeIconList = iBmp.CreateFileFromResourceBitmap("SwCSharpGetSubAssyMassProps.ToolbarLarge.bmp", thisAssembly);
            cmdGroup.SmallIconList = iBmp.CreateFileFromResourceBitmap("SwCSharpGetSubAssyMassProps.ToolbarSmall.bmp", thisAssembly);
            cmdGroup.LargeMainIcon = iBmp.CreateFileFromResourceBitmap("SwCSharpGetSubAssyMassProps.MainIconLarge.bmp", thisAssembly);
            cmdGroup.SmallMainIcon = iBmp.CreateFileFromResourceBitmap("SwCSharpGetSubAssyMassProps.MainIconSmall.bmp", thisAssembly);

            cmdGroup.AddCommandItem("Get Mass", -1, "Get Mass", "Get Mass", 0, "GetMass", "", 0);

            cmdGroup.HasToolbar = true;
            cmdGroup.HasMenu = true;
            cmdGroup.Activate();

            thisAssembly = null;
            iBmp.Dispose();
        }


        public void RemoveCommandMgr()
        {
            iCmdMgr.RemoveCommandGroup(1);
        }

        #endregion

        #region UI Callbacks
        public void GetMass()
        {
            this.swBodies = new List<IBody2>();

            try
            {
                IModelDoc2 swModel;
                IModelDocExtension swDocExt;
                IMassProperty swMass;
                ISelectionMgr swSelMgr;
                IComponent2 swComp;

                swModel = this.iSwApp.IActiveDoc2;
                swDocExt = swModel.Extension;
                swMass = swDocExt.CreateMassProperty();
                swSelMgr = swModel.ISelectionManager;
                swComp = (Component2)swSelMgr.GetSelectedObjectsComponent2(1);
                GetCompBodies(swComp);
                bool bRet = swMass.AddBodies(this.swBodies);

                this.iSwApp.SendMsgToUser("Bodies: " + this.swBodies.Count + "; Overall Mass: " + swMass.Mass.ToString());
                
            }
            catch (Exception e)
            {
                
            } 
        }

        private void GetCompBodies(IComponent2 swComp) 
        {
            object[] child = (object[])swComp.GetChildren();

            if (child.Length == 0)
            {                
                object[] bodies = (object[])swComp.GetBodies2((int)swBodyType_e.swSolidBody);

                for (int j = 0; j < bodies.Length; j++) 
                {
                    this.swBodies.Add(bodies[j] as IBody2);
                }
            }
            else 
            {
                for (int i = 0; i < child.Length; i++) 
                {
                    GetCompBodies(child[i] as IComponent2);
                }
            }
        }

        #endregion

    }

    #region SwAddin class attributes
    [AttributeUsage(AttributeTargets.Class)]
    public class SwAddinAttribute : System.Attribute
    {
        public string Description = "";
        public string Title = "";
        public bool LoadAtStartup = false;

        public SwAddinAttribute()
        {
        }
    }
    #endregion
}
